@@ -70,22 +70,35 @@ class ApplicationConfigForm extends Form
             )
         );
 
-        $this->addElement(
-            'select',
-            'global_config_backend',
-            array(
-                'required'      => true,
-                'autosubmit'    => true,
-                'label'         => $this->translate('User Preference Storage Type'),
-                'multiOptions'  => array(
-                    'ini'   => $this->translate('File System (INI Files)'),
-                    'db'    => $this->translate('Database'),
-                    'none'  => $this->translate('Don\'t Store Preferences')
-                )
-            )
-        );
+        // we do not need this form for setup because we set the database there as default.
+        // this form is only displayed in configuration -> application if preferences backend type of ini is recognized
+        if (isset($formData['global_config_backend']) && $formData['global_config_backend'] === 'ini') {
+            $this->addElement(
+                'select',
+                'global_config_backend',
+                [
+                    'required'      => true,
+                    'autosubmit'    => true,
+                    'label'         => $this->translate('User Preference Storage Type'),
+                    'multiOptions'  => [
+                        'ini'   => $this->translate('File System (INI Files)'),
+                        'db'    => $this->translate('Database')
+                    ]
+                ]
+            );
+        } else {
+            $this->addElement(
+                'hidden',
+                'global_config_backend',
+                [
+                    'required'  => true,
+                    'value'     => 'db',
+                    'disabled'  => true
+                ]
+            );
+        }
 
-        if (isset($formData['global_config_backend']) && $formData['global_config_backend'] === 'db') {
+        if (! isset($formData['global_config_backend']) || $formData['global_config_backend'] === 'db') {
             $backends = array();
             foreach (ResourceFactory::getResourceConfigs()->toArray() as $name => $resource) {
                 if ($resource['type'] === 'db') {
@@ -98,8 +111,13 @@ class ApplicationConfigForm extends Form
                 'global_config_resource',
                 array(
                     'required'      => true,
-                    'multiOptions'  => $backends,
-                    'label'         => $this->translate('Database Connection')
+                    'multiOptions'  => array_merge(
+                        ['' => sprintf(' - %s - ', $this->translate('Please choose'))],
+                        $backends
+                    ),
+                    'disable'       => [''],
+                    'value'         => '',
+                    'label'         => $this->translate('Configuration Database')
                 )
             );
         }
