@@ -37,6 +37,13 @@ class UserBackendConfigForm extends ConfigForm
      */
     protected $backendToLoad;
 
+    /**
+     * The loaded custom backends list
+     *
+     * @var array
+     */
+    protected $customBackends;
+
     /**
      * Initialize this form
      */
@@ -45,6 +52,7 @@ class UserBackendConfigForm extends ConfigForm
         $this->setName('form_config_authbackend');
         $this->setSubmitLabel($this->translate('Save Changes'));
         $this->setValidatePartial(true);
+        $this->customBackends = UserBackend::getCustomBackendConfigForms();
     }
 
     /**
@@ -114,6 +122,10 @@ class UserBackendConfigForm extends ConfigForm
                 $form = new ExternalBackendForm();
                 break;
             default:
+                if (isset($this->customBackends[$type])) {
+                    return new $this->customBackends[$type]();
+                }
+
                 throw new InvalidArgumentException(
                     sprintf($this->translate('Invalid backend type "%s" provided'), $type)
                 );
@@ -274,6 +286,9 @@ class UserBackendConfigForm extends ConfigForm
             $backendTypes['external'] = $this->translate('External');
         }
 
+        $customBackendTypes = array_keys($this->customBackends);
+        $backendTypes += array_combine($customBackendTypes, $customBackendTypes);
+
         if ($backendType === null) {
             $backendType = key($backendTypes);
         }
