@@ -3,8 +3,8 @@
 
 namespace Icinga\Forms;
 
-use Exception;
 use DateTimeZone;
+use Exception;
 use Icinga\Application\Config;
 use Icinga\Application\Icinga;
 use Icinga\Application\Logger;
@@ -12,12 +12,14 @@ use Icinga\Authentication\Auth;
 use Icinga\User\Preferences;
 use Icinga\User\Preferences\PreferencesStore;
 use Icinga\Util\TimezoneDetect;
-use Icinga\Util\Translator;
-use Icinga\Web\Cookie;
 use Icinga\Web\Form;
 use Icinga\Web\Notification;
 use Icinga\Web\Session;
 use Icinga\Web\StyleSheet;
+use ipl\Html\HtmlElement;
+use ipl\I18n\GettextTranslator;
+use ipl\I18n\Locale;
+use ipl\I18n\StaticTranslator;
 
 /**
  * Form class to adjust user preferences
@@ -44,6 +46,7 @@ class PreferenceForm extends Form
     public function init()
     {
         $this->setName('form_config_preferences');
+        $this->setSubmitLabel($this->translate('Save to the Preferences'));
     }
 
     /**
@@ -90,15 +93,19 @@ class PreferenceForm extends Form
      */
     public function onSuccess()
     {
+        $currentPreferences = $this->Auth()->getUser()->getPreferences();
+        $oldTheme = $currentPreferences->getValue('icingaweb', 'theme');
+        $oldMode = $currentPreferences->getValue('icingaweb', 'theme_mode');
+        $oldLocale = $currentPreferences->getValue('icingaweb', 'language');
+        $defaultTheme = Config::app()->get('themes', 'default', StyleSheet::DEFAULT_THEME);
+
         $this->preferences = new Preferences($this->store ? $this->store->load() : array());
-
-        $oldTheme = $this->preferences->getValue('icingaweb', 'theme');
-
         $webPreferences = $this->preferences->get('icingaweb', array());
         foreach ($this->getValues() as $key => $value) {
             if ($value === ''
+                || $value === null
                 || $value === 'autodetect'
-                || ($key === 'theme' && $value === Config::app()->get('themes', 'default', StyleSheet::DEFAULT_THEME))
+                || ($key === 'theme' && $value === $defaultTheme)
             ) {
                 if (isset($webPreferences[$key])) {
                     unset($webPreferences[$key]);
@@ -108,17 +115,26 @@ class PreferenceForm extends Form
             }
         }
         $this->preferences->icingaweb = $webPreferences;
-
         Session::getSession()->user->setPreferences($this->preferences);
 
-        if (($theme = $this->getElement('theme')) !== null
+        if ((($theme = $this->getElement('theme')) !== null
             && ($theme = $theme->getValue()) !== $oldTheme
+            && ($theme !== $defaultTheme || $oldTheme !== null))
+            || (($mode = $this->getElement('theme_mode')) !== null
+            && ($mode->getValue()) !== $oldMode)
         ) {
             $this->getResponse()->setReloadCss(true);
         }
 
+        if (($locale = $this->getElement('language')) !== null
+            && $locale->getValue() !== 'autodetect'
+            && $locale->getValue() !== $oldLocale
+        ) {
+            $this->getResponse()->setHeader('X-Icinga-Redirect-Http', 'yes');
+        }
+
         try {
-            if ($this->store && $this->getElement('btn_submit_preferences')->isChecked()) {
+            if ($this->store && $this->getElement('btn_submit')->isChecked()) {
                 $this->save();
                 Notification::success($this->translate('Preferences successfully saved'));
             } else {
@@ -148,6 +164,10 @@ class PreferenceForm extends Form
             $values['timezone'] = 'autodetect';
         }
 
+        if (! isset($values['auto_refresh'])) {
+            $values['auto_refresh'] = '1';
+        }
+
         $this->populate($values);
     }
 
@@ -179,6 +199,7 @@ class PreferenceForm extends Form
                     array(
                         'label'         => $this->translate('Theme', 'Form element label'),
                         'multiOptions'  => $themes,
+                        'autosubmit'    => true,
                         'value'         => $this->preferences->getValue(
                             'icingaweb',
                             'theme',
@@ -189,14 +210,57 @@ class PreferenceForm extends Form
             }
         }
 
-        $languages = array();
+        if (isset($formData['theme']) && $formData['theme'] !== StyleSheet::DEFAULT_THEME) {
+            $themeFile = StyleSheet::getThemeFile($formData['theme']);
+            $file = $themeFile !== null ? @file_get_contents($themeFile) : false;
+            if ($file && strpos($file, StyleSheet::LIGHT_MODE_IDENTIFIER) === false) {
+                $disabled = ['', 'light', 'system'];
+            }
+        }
 
-        $locale = $this->getLocale();
+        $this->addElement(
+            'radio',
+            'theme_mode',
+            [
+                'class' => 'theme-mode-input',
+                'label' => $this->translate('Theme Mode'),
+                'multiOptions' => [
+                    '' => HtmlElement::create(
+                        'img',
+                        ['src' => $this->getView()->href('img/theme-mode-thumbnail-dark.svg')]
+                    ) . HtmlElement::create('span', [], $this->translate('Dark')),
+                    'light' => HtmlElement::create(
+                        'img',
+                        ['src' => $this->getView()->href('img/theme-mode-thumbnail-light.svg')]
+                    ) . HtmlElement::create('span', [], $this->translate('Light')),
+                    'system' => HtmlElement::create(
+                        'img',
+                        ['src' => $this->getView()->href('img/theme-mode-thumbnail-system.svg')]
+                    ) . HtmlElement::create('span', [], $this->translate('System'))
+                ],
+                'value' => isset($value) ? $value : '',
+                'disable' => isset($disabled) ? $disabled : [],
+                'escape' => false,
+                'decorators' => array_merge(
+                    array_slice(self::$defaultElementDecorators, 0, -1),
+                    [['HtmlTag', ['tag' => 'div', 'class' => 'control-group theme-mode']]]
+                )
+            ]
+        );
+
+        /** @var GettextTranslator $translator */
+        $translator = StaticTranslator::$instance;
+
+        $languages = array();
+        $availableLocales = $translator->listLocales();
+
+        $locale = $this->getLocale($availableLocales);
         if ($locale !== null) {
             $languages['autodetect'] = sprintf($this->translate('Browser (%s)', 'preferences.form'), $locale);
         }
 
-        foreach (Translator::getAvailableLocaleCodes() as $language) {
+        $availableLocales[] = $translator->getDefaultLocale();
+        foreach ($availableLocales as $language) {
             $languages[$language] = $language;
         }
 
@@ -250,12 +314,11 @@ class PreferenceForm extends Form
             )
         );
 
-        if (Auth::getInstance()->hasPermission('application/stacktraces')) {
+        if (Auth::getInstance()->hasPermission('user/application/stacktraces')) {
             $this->addElement(
                 'checkbox',
                 'show_stacktraces',
                 array(
-                    'required'      => true,
                     'value'         => $this->getDefaultShowStacktraces(),
                     'label'         => $this->translate('Show Stacktraces'),
                     'description'   => $this->translate('Set whether to show an exception\'s stacktrace.')
@@ -267,7 +330,6 @@ class PreferenceForm extends Form
             'checkbox',
             'show_benchmark',
             array(
-                'required'  => true,
                 'label'     => $this->translate('Use benchmark')
             )
         );
@@ -277,6 +339,7 @@ class PreferenceForm extends Form
             'auto_refresh',
             array(
                 'required'      => false,
+                'autosubmit'    => true,
                 'label'         => $this->translate('Enable auto refresh'),
                 'description'   => $this->translate(
                     'This option allows you to enable or to disable the global page content auto refresh'
@@ -285,12 +348,35 @@ class PreferenceForm extends Form
             )
         );
 
+        if (isset($formData['auto_refresh']) && $formData['auto_refresh']) {
+            $this->addElement(
+                'select',
+                'auto_refresh_speed',
+                [
+                    'required'      => false,
+                    'label'         => $this->translate('Auto refresh speed'),
+                    'description'   => $this->translate(
+                        'This option allows you to speed up or to slow down the global page content auto refresh'
+                    ),
+                    'multiOptions'  => [
+                        '0.5'   => $this->translate('Fast', 'refresh_speed'),
+                        ''      => $this->translate('Default', 'refresh_speed'),
+                        '2'     => $this->translate('Moderate', 'refresh_speed'),
+                        '4'     => $this->translate('Slow', 'refresh_speed')
+                    ],
+                    'value'         => ''
+                ]
+            );
+        }
+
         $this->addElement(
             'number',
             'default_page_size',
             array(
                 'label'         => $this->translate('Default page size'),
                 'description'   => $this->translate('Default number of items per page for list views'),
+                'placeholder'   => 25,
+                'min'           => 25,
                 'step'          => 1
             )
         );
@@ -298,7 +384,7 @@ class PreferenceForm extends Form
         if ($this->store) {
             $this->addElement(
                 'submit',
-                'btn_submit_preferences',
+                'btn_submit',
                 array(
                     'ignore'        => true,
                     'label'         => $this->translate('Save to the Preferences'),
@@ -331,7 +417,7 @@ class PreferenceForm extends Form
         );
 
         $this->addDisplayGroup(
-            array('btn_submit_preferences', 'btn_submit_session', 'preferences-progress'),
+            array('btn_submit', 'btn_submit_session', 'preferences-progress'),
             'submit_buttons',
             array(
                 'decorators' => array(
@@ -342,6 +428,20 @@ class PreferenceForm extends Form
         );
     }
 
+    public function addSubmitButton()
+    {
+        return $this;
+    }
+
+    public function isSubmitted()
+    {
+        if (parent::isSubmitted()) {
+            return true;
+        }
+
+        return $this->getElement('btn_submit_session')->isChecked();
+    }
+
     /**
      * Return the current default timezone
      *
@@ -362,10 +462,10 @@ class PreferenceForm extends Form
      *
      * @return string|null
      */
-    protected function getLocale()
+    protected function getLocale($availableLocales)
     {
         return isset($_SERVER['HTTP_ACCEPT_LANGUAGE'])
-            ? Translator::getPreferredLocaleCode($_SERVER['HTTP_ACCEPT_LANGUAGE'])
+            ? (new Locale())->getPreferred($_SERVER['HTTP_ACCEPT_LANGUAGE'], $availableLocales)
             : null;
     }
 
