@@ -4,24 +4,23 @@
 namespace Icinga\Application\Modules;
 
 use Exception;
-use Zend_Controller_Router_Route;
-use Zend_Controller_Router_Route_Abstract;
-use Zend_Controller_Router_Route_Regex;
 use Icinga\Application\ApplicationBootstrap;
 use Icinga\Application\Config;
+use Icinga\Application\Hook;
 use Icinga\Application\Icinga;
 use Icinga\Application\Logger;
-use Icinga\Application\Modules\DashboardContainer;
-use Icinga\Application\Modules\MenuItemContainer;
 use Icinga\Exception\IcingaException;
 use Icinga\Exception\ProgrammingError;
 use Icinga\Module\Setup\SetupWizard;
 use Icinga\Util\File;
-use Icinga\Util\Translator;
-use Icinga\Web\Controller\Dispatcher;
-use Icinga\Application\Hook;
 use Icinga\Web\Navigation\Navigation;
 use Icinga\Web\Widget;
+use ipl\I18n\GettextTranslator;
+use ipl\I18n\StaticTranslator;
+use ipl\I18n\Translation;
+use Zend_Controller_Router_Route;
+use Zend_Controller_Router_Route_Abstract;
+use Zend_Controller_Router_Route_Regex;
 
 /**
  * Module handling
@@ -30,6 +29,11 @@ use Icinga\Web\Widget;
  */
 class Module
 {
+    use Translation {
+        translate as protected;
+        translatePlural as protected;
+    }
+
     /**
      * Module name
      *
@@ -51,6 +55,13 @@ class Module
      */
     private $cssdir;
 
+    /**
+     * Directory for Javascript
+     *
+     * @var string
+     */
+    private $jsdir;
+
     /**
      * Base application directory
      *
@@ -258,6 +269,8 @@ class Module
         $this->runScript      = $basedir . '/run.php';
         $this->configScript   = $basedir . '/configuration.php';
         $this->metadataFile   = $basedir . '/module.info';
+
+        $this->translationDomain = $name;
     }
 
     /**
@@ -315,10 +328,11 @@ class Module
         foreach ($panes as $pane) {
             /** @var DashboardContainer $pane */
             $dashlets = [];
-            foreach ($pane->getDashlets() as $dashletName => $dashletUrl) {
+            foreach ($pane->getDashlets() as $dashletName => $dashletConfig) {
                 $dashlets[$dashletName] = [
-                    'label' => $this->translate($dashletName),
-                    'url'   => $dashletUrl
+                    'label'     => $this->translate($dashletName),
+                    'url'       => $dashletConfig['url'],
+                    'priority'  => $dashletConfig['priority']
                 ];
             }
 
@@ -468,7 +482,7 @@ class Module
      * @param string $name
      * @param bool   $autoload
      *
-     * @return mixed
+     * @return  self
      *
      * @throws ProgrammingError When the module is not yet loaded
      */
@@ -642,12 +656,89 @@ class Module
      * Get the module dependencies
      *
      * @return array
+     * @deprecated Use method getRequiredModules() instead
      */
     public function getDependencies()
     {
         return $this->metadata()->depends;
     }
 
+    /**
+     * Get required libraries
+     *
+     * @return array
+     */
+    public function getRequiredLibraries()
+    {
+        $requiredLibraries = $this->metadata()->libraries;
+
+        // Register module requirements for ipl and reactbundle as library requirements
+        $requiredModules = $this->metadata()->modules ?: $this->metadata()->depends;
+        if (isset($requiredModules['ipl']) && ! isset($requiredLibraries['icinga-php-library'])) {
+            $requiredLibraries['icinga-php-library'] = $requiredModules['ipl'];
+        }
+
+        if (isset($requiredModules['reactbundle']) && ! isset($requiredLibraries['icinga-php-thirdparty'])) {
+            $requiredLibraries['icinga-php-thirdparty'] = $requiredModules['reactbundle'];
+        }
+
+        return $requiredLibraries;
+    }
+
+    /**
+     * Get required modules
+     *
+     * @return array
+     */
+    public function getRequiredModules()
+    {
+        $requiredModules = $this->metadata()->modules ?: $this->metadata()->depends;
+
+        $hasIcingadb = isset($requiredModules['icingadb']);
+        if (isset($requiredModules['monitoring']) && ($this->isSupportingIcingadb() || $hasIcingadb)) {
+            $requiredMods = [];
+            $icingadbVersion = true;
+            if ($hasIcingadb) {
+                $icingadbVersion = isset($requiredModules['icingadb']) ? $requiredModules['icingadb'] : true;
+                unset($requiredModules['icingadb']);
+            }
+
+            foreach ($requiredModules as $name => $version) {
+                $requiredMods[$name] = $version;
+                if ($name === 'monitoring') {
+                    $requiredMods['icingadb'] = $icingadbVersion;
+                }
+            }
+
+            $requiredModules = $requiredMods;
+        }
+
+        // Both modules are deprecated and their successors are now dependencies of web itself
+        unset($requiredModules['ipl'], $requiredModules['reactbundle']);
+
+        return $requiredModules;
+    }
+
+    /**
+     * Check whether module supports icingadb
+     *
+     * @return bool
+     */
+    protected function isSupportingIcingadb()
+    {
+        $icingadbSupportingModules = [
+            'cube'              => '1.2.0',
+            'jira'              => '1.2.0',
+            'graphite'          => '1.2.0',
+            'director'          => '1.9.0',
+            'toplevelview'      => '0.4.0',
+            'businessprocess'   => '2.4.0'
+        ];
+
+        return array_key_exists($this->getName(), $icingadbSupportingModules)
+            && version_compare($this->getVersion(), $icingadbSupportingModules[$this->getName()], '>=');
+    }
+
     /**
      * Fetch module metadata
      *
@@ -656,18 +747,21 @@ class Module
     protected function metadata()
     {
         if ($this->metadata === null) {
-            $metadata = (object) array(
-                'name'        => $this->getName(),
-                'version'     => '0.0.0',
-                'title'       => null,
-                'description' => '',
-                'depends'     => array(),
-            );
+            $metadata = (object) [
+                'name'          => $this->getName(),
+                'version'       => '0.0.0',
+                'title'         => null,
+                'description'   => '',
+                'depends'       => [],
+                'libraries'     => [],
+                'modules'       => []
+            ];
 
             if (file_exists($this->metadataFile)) {
                 $key = null;
+                $simpleRequires = false;
                 $file = new File($this->metadataFile, 'r');
-                foreach ($file as $line) {
+                foreach ($file as $lineno => $line) {
                     $line = rtrim($line);
 
                     if ($key === 'description') {
@@ -682,27 +776,64 @@ class Module
                         continue;
                     }
 
-                    list($key, $val) = preg_split('/:\s+/', $line, 2);
-                    $key = lcfirst($key);
+                    if (strpos($line, ':') === false) {
+                        Logger::debug(
+                            "Can't process line %d in %s: Line does not specify a key:value pair"
+                            . " nor is it part of the description (indented with a single space)",
+                            $lineno,
+                            $this->metadataFile
+                        );
 
+                        break;
+                    }
+
+                    $parts = preg_split('/:\s+/', $line, 2);
+                    if (count($parts) === 1) {
+                        $parts[] = '';
+                    }
+
+                    list($key, $val) = $parts;
+
+                    $key = strtolower($key);
                     switch ($key) {
+                        case 'requires':
+                            if ($val) {
+                                $simpleRequires = true;
+                                $key = 'libraries';
+                            } else {
+                                break;
+                            }
+
+                            // Shares the syntax with `Depends`
+                        case '  libraries':
+                        case '  modules':
+                            if ($simpleRequires && $key[0] === ' ') {
+                                Logger::debug(
+                                    'Can\'t process line %d in %s: Requirements already registered by a previous line',
+                                    $lineno,
+                                    $this->metadataFile
+                                );
+                                break;
+                            }
+
+                            $key = ltrim($key);
+                            // Shares the syntax with `Depends`
                         case 'depends':
                             if (strpos($val, ' ') === false) {
-                                $metadata->depends[$val] = true;
+                                $metadata->{$key}[$val] = true;
                                 continue 2;
                             }
 
                             $parts = preg_split('/,\s+/', $val);
                             foreach ($parts as $part) {
-                                if (preg_match('/^(\w+)\s+\((.+)\)$/', $part, $m)) {
-                                    $metadata->depends[$m[1]] = $m[2];
+                                if (preg_match('/^([\w\-\/]+)\s+\((.+)\)$/', $part, $m)) {
+                                    $metadata->{$key}[$m[1]] = $m[2];
                                 } else {
-                                    // TODO: FAIL?
-                                    continue;
+                                    $metadata->{$key}[$part] = true;
                                 }
                             }
-                            break;
 
+                            break;
                         case 'description':
                             if ($metadata->title === null) {
                                 $metadata->title = $val;
@@ -722,8 +853,6 @@ class Module
             }
 
             if ($metadata->description === '') {
-                // TODO: Check whether the translation module is able to
-                //       extract this
                 $metadata->description = t(
                     'This module has no description'
                 );
@@ -744,6 +873,16 @@ class Module
         return $this->cssdir;
     }
 
+    /**
+     * Get the module's JS directory
+     *
+     * @return string
+     */
+    public function getJsDir()
+    {
+        return $this->jsdir;
+    }
+
     /**
      * Get the module's controller directory
      *
@@ -1102,9 +1241,10 @@ class Module
      */
     protected function registerLocales()
     {
-        if ($this->hasLocales()) {
-            Translator::registerDomain($this->name, $this->localedir);
+        if ($this->hasLocales() && StaticTranslator::$instance instanceof GettextTranslator) {
+            StaticTranslator::$instance->addTranslationDirectory($this->localedir, $this->name);
         }
+
         return $this;
     }
 
@@ -1317,22 +1457,4 @@ class Module
         $this->routes[$name] = $route;
         return $this;
     }
-
-    /**
-     * (non-PHPDoc)
-     * @see Translator::translate() For the function documentation.
-     */
-    protected function translate($string, $context = null)
-    {
-        return mt($this->name, $string, $context);
-    }
-
-    /**
-     * (non-PHPDoc)
-     * @see Translator::translatePlural() For the function documentation.
-     */
-    protected function translatePlural($textSingular, $textPlural, $number, $context = null)
-    {
-        return mtp($this->name, $textSingular, $textPlural, $number, $context);
-    }
 }
