@@ -17,6 +17,7 @@ use Icinga\User;
 use Icinga\User\Preferences;
 use Icinga\User\Preferences\PreferencesStore;
 use Icinga\Web\Session;
+use Icinga\Web\StyleSheet;
 
 class Auth
 {
@@ -98,97 +99,36 @@ class Auth
 
     public function setAuthenticated(User $user, $persist = true)
     {
-        $username = $user->getUsername();
-        try {
-            $config = Config::app();
-        } catch (NotReadableError $e) {
-            Logger::error(
-                new IcingaException(
-                    'Cannot load preferences for user "%s". An exception was thrown: %s',
-                    $username,
-                    $e
-                )
-            );
-            $config = new Config();
-        }
-        if ($config->get('global', 'config_backend', 'ini') !== 'none') {
-            $preferencesConfig = new ConfigObject(array(
-                'store'     => $config->get('global', 'config_backend', 'ini'),
-                'resource'  => $config->get('global', 'config_resource')
-            ));
-            try {
-                $preferencesStore = PreferencesStore::create(
-                    $preferencesConfig,
-                    $user
-                );
-                $preferences = new Preferences($preferencesStore->load());
-            } catch (Exception $e) {
-                Logger::error(
-                    new IcingaException(
-                        'Cannot load preferences for user "%s". An exception was thrown: %s',
-                        $username,
-                        $e
-                    )
-                );
-                $preferences = new Preferences();
-            }
-        } else {
-            $preferences = new Preferences();
-        }
-        // TODO(el): Quick-fix for #10957. Only reload CSS if the theme changed.
-        $this->getResponse()->setReloadCss(true);
-        $user->setPreferences($preferences);
-        $groups = $user->getGroups();
-        $userBackendName = $user->getAdditional('backend_name');
-        foreach (Config::app('groups') as $name => $config) {
-            $groupsUserBackend = $config->user_backend;
-            if ($groupsUserBackend
-                && $groupsUserBackend !== 'none'
-                && $userBackendName !== null
-                && $groupsUserBackend !== $userBackendName
-            ) {
-                // Do not ask for Group membership if a specific User Backend
-                // has been assigned to that Group Backend, and the user has
-                // been authenticated by another User Backend
-                continue;
-            }
+        $this->setupUser($user);
 
-            try {
-                $groupBackend = UserGroupBackend::create($name, $config);
-                $groupsFromBackend = $groupBackend->getMemberships($user);
-            } catch (Exception $e) {
-                Logger::error(
-                    'Can\'t get group memberships for user \'%s\' from backend \'%s\'. An exception was thrown: %s',
-                    $username,
-                    $name,
-                    $e
-                );
-                continue;
+        // Reload CSS if the theme changed
+        $themingConfig = Icinga::app()->getConfig()->getSection('themes');
+        $userTheme = $user->getPreferences()->getValue('icingaweb', 'theme');
+        if (! (bool) $themingConfig->get('disabled', false) && $userTheme !== null) {
+            $defaultTheme = $themingConfig->get('default', StyleSheet::DEFAULT_THEME);
+            if ($userTheme !== $defaultTheme) {
+                $this->getResponse()->setReloadCss(true);
             }
-            if (empty($groupsFromBackend)) {
-                Logger::debug(
-                    'No groups found in backend "%s" which the user "%s" is a member of.',
-                    $name,
-                    $user->getUsername()
-                );
-                continue;
-            }
-            $groupsFromBackend = array_values($groupsFromBackend);
-            Logger::debug(
-                'Groups found in backend "%s" for user "%s": %s',
-                $name,
-                $user->getUsername(),
-                join(', ', $groupsFromBackend)
-            );
-            $groups = array_merge($groups, array_combine($groupsFromBackend, $groupsFromBackend));
         }
-        $user->setGroups($groups);
-        $admissionLoader = new AdmissionLoader();
-        $admissionLoader->applyRoles($user);
+
+        // Also reload CSS if the theme mode changed
+        $themeMode = $user->getPreferences()->getValue('icingaweb', 'theme_mode');
+        if ($themeMode && $themeMode !== StyleSheet::DEFAULT_MODE) {
+            $this->getResponse()->setReloadCss(true);
+        }
+
+        // Reload entire layout if the locale changed
+        if (($locale = $user->getPreferences()->getValue('icingaweb', 'language')) !== null) {
+            if (setlocale(LC_ALL, 0) !== $locale && $this->getRequest()->isXmlHttpRequest()) {
+                $this->getResponse()->setHeader('X-Icinga-Redirect-Http', 'yes');
+            }
+        }
+
         $this->user = $user;
         if ($persist) {
             $this->persistCurrentUser();
         }
+
         AuditHook::logActivity('login', 'User logged in');
     }
 
@@ -409,4 +349,110 @@ class Auth
         $this->user = null;
         Session::getSession()->purge();
     }
+
+    /**
+     * Setup the given user
+     *
+     * This loads preferences, groups and roles.
+     *
+     * @param User $user
+     *
+     * @return void
+     */
+    public function setupUser(User $user)
+    {
+        // Load the user's preferences
+
+        try {
+            $config = Config::app();
+        } catch (NotReadableError $e) {
+            Logger::error(
+                new IcingaException(
+                    'Cannot load preferences for user "%s". An exception was thrown: %s',
+                    $user->getUsername(),
+                    $e
+                )
+            );
+            $config = new Config();
+        }
+
+        if ($config->get('global', 'config_backend', 'db') !== 'none') {
+            $preferencesConfig = new ConfigObject([
+                'store'     => $config->get('global', 'config_backend', 'db'),
+                'resource'  => $config->get('global', 'config_resource')
+            ]);
+
+            try {
+                $preferencesStore = PreferencesStore::create($preferencesConfig, $user);
+                $preferences = new Preferences($preferencesStore->load());
+            } catch (Exception $e) {
+                Logger::error(
+                    new IcingaException(
+                        'Cannot load preferences for user "%s". An exception was thrown: %s',
+                        $user->getUsername(),
+                        $e
+                    )
+                );
+                $preferences = new Preferences();
+            }
+        } else {
+            $preferences = new Preferences();
+        }
+
+        $user->setPreferences($preferences);
+
+        // Load the user's groups
+        $groups = $user->getGroups();
+        $userBackendName = $user->getAdditional('backend_name');
+        foreach (Config::app('groups') as $name => $config) {
+            $groupsUserBackend = $config->user_backend;
+            if ($groupsUserBackend
+                && $groupsUserBackend !== 'none'
+                && $userBackendName !== null
+                && $groupsUserBackend !== $userBackendName
+            ) {
+                // Do not ask for Group membership if a specific User Backend
+                // has been assigned to that Group Backend, and the user has
+                // been authenticated by another User Backend
+                continue;
+            }
+
+            try {
+                $groupBackend = UserGroupBackend::create($name, $config);
+                $groupsFromBackend = $groupBackend->getMemberships($user);
+            } catch (Exception $e) {
+                Logger::error(
+                    'Can\'t get group memberships for user \'%s\' from backend \'%s\'. An exception was thrown: %s',
+                    $user->getUsername(),
+                    $name,
+                    $e
+                );
+                continue;
+            }
+
+            if (empty($groupsFromBackend)) {
+                Logger::debug(
+                    'No groups found in backend "%s" which the user "%s" is a member of.',
+                    $name,
+                    $user->getUsername()
+                );
+                continue;
+            }
+
+            $groupsFromBackend = array_values($groupsFromBackend);
+            Logger::debug(
+                'Groups found in backend "%s" for user "%s": %s',
+                $name,
+                $user->getUsername(),
+                join(', ', $groupsFromBackend)
+            );
+            $groups = array_merge($groups, array_combine($groupsFromBackend, $groupsFromBackend));
+        }
+
+        $user->setGroups($groups);
+
+        // Load the user's roles
+        $admissionLoader = new AdmissionLoader();
+        $admissionLoader->applyRoles($user);
+    }
 }
