@@ -21,9 +21,10 @@ namespace {
 namespace Icinga\Test {
 
     use Exception;
+    use ipl\I18n\NoopTranslator;
+    use ipl\I18n\StaticTranslator;
     use RuntimeException;
     use Mockery;
-    use PHPUnit_Framework_TestCase;
     use Icinga\Application\Icinga;
     use Icinga\Data\ConfigObject;
     use Icinga\Data\ResourceFactory;
@@ -32,7 +33,7 @@ namespace Icinga\Test {
     /**
      * Class BaseTestCase
      */
-    abstract class BaseTestCase extends PHPUnit_Framework_TestCase implements DbTest
+    abstract class BaseTestCase extends Mockery\Adapter\Phpunit\MockeryTestCase implements DbTest
     {
         /**
          * Path to application/
@@ -138,9 +139,11 @@ namespace Icinga\Test {
         /**
          * Setup MVC bootstrapping and ensure that the Icinga-Mock gets reinitialized
          */
-        public function setUp()
+        public function setUp(): void
         {
             parent::setUp();
+
+            StaticTranslator::$instance = new NoopTranslator();
             $this->setupIcingaMock();
         }
 
@@ -334,6 +337,23 @@ namespace Icinga\Test {
                 $adapter->exec('DROP TABLE ' . $table . ';');
             }
         }
+
+        /**
+         * Add assertMatchesRegularExpression() method for phpunit >= 8.0 < 9.0 for compatibility with PHP 7.2.
+         *
+         * @TODO Remove once PHP 7.2 support is not needed for testing anymore.
+         */
+        public static function assertMatchesRegularExpression(
+            string $pattern,
+            string $string,
+            string $message = ''
+        ): void {
+            if (method_exists(parent::class, 'assertMatchesRegularExpression')) {
+                parent::assertMatchesRegularExpression($pattern, $string, $message);
+            } else {
+                static::assertRegExp($pattern, $string, $message);
+            }
+        }
     }
 
     BaseTestCase::setupTimezone();
