@@ -3,6 +3,10 @@
 
 namespace Icinga\Web\Controller;
 
+use Icinga\Application\Modules\Module;
+use Icinga\Common\PdfExport;
+use Icinga\File\Pdf;
+use ipl\I18n\Translation;
 use Zend_Controller_Action;
 use Zend_Controller_Action_HelperBroker;
 use Zend_Controller_Request_Abstract;
@@ -13,10 +17,8 @@ use Icinga\Authentication\Auth;
 use Icinga\Exception\Http\HttpMethodNotAllowedException;
 use Icinga\Exception\IcingaException;
 use Icinga\Exception\ProgrammingError;
-use Icinga\File\Pdf;
 use Icinga\Forms\AutoRefreshForm;
 use Icinga\Security\SecurityException;
-use Icinga\Util\Translator;
 use Icinga\Web\Session;
 use Icinga\Web\Url;
 use Icinga\Web\UrlParams;
@@ -40,6 +42,11 @@ use Icinga\Web\Window;
  */
 class ActionController extends Zend_Controller_Action
 {
+    use Translation;
+    use PdfExport {
+        sendAsPdf as private newSendAsPdf;
+    }
+
     /**
      * The login route to use when requiring authentication
      */
@@ -64,6 +71,13 @@ class ActionController extends Zend_Controller_Action
      */
     protected $moduleName;
 
+    /**
+     * A page's automatic refresh interval
+     *
+     * The initial value will not be subject to a user's preferences.
+     *
+     * @var int
+     */
     protected $autorefreshInterval;
 
     protected $reloadCss = false;
@@ -122,15 +136,19 @@ class ActionController extends Zend_Controller_Action
             ->_setInvokeArgs($invokeArgs);
         $this->_helper = new Zend_Controller_Action_HelperBroker($this);
 
-        $this->handlerBrowserWindows();
         $moduleName = $this->getModuleName();
         $this->view->defaultTitle = static::DEFAULT_TITLE;
-        $this->view->translationDomain = $moduleName !== 'default' ? $moduleName : 'icinga';
+        $this->translationDomain = $moduleName !== 'default' ? $moduleName : 'icinga';
+        $this->view->translationDomain = $this->translationDomain;
         $this->_helper->layout()->isIframe = $request->getUrl()->shift('isIframe');
         $this->_helper->layout()->showFullscreen = $request->getUrl()->shift('showFullscreen');
         $this->_helper->layout()->moduleName = $moduleName;
 
-        $this->view->compact = $request->getParam('view') === 'compact';
+        $this->view->compact = false;
+        if ($request->getUrl()->getParam('view') === 'compact') {
+            $request->getUrl()->remove('view');
+            $this->view->compact = true;
+        }
         if ($request->getUrl()->shift('showCompact')) {
             $this->view->compact = true;
         }
@@ -229,25 +247,12 @@ class ActionController extends Zend_Controller_Action
     public function Window()
     {
         if ($this->window === null) {
-            $this->window = new Window(
-                $this->_request->getHeader('X-Icinga-WindowId', Window::UNDEFINED)
-            );
+            $this->window = Window::getInstance();
         }
+
         return $this->window;
     }
 
-    protected function handlerBrowserWindows()
-    {
-        if ($this->isXhr()) {
-            $id = $this->_request->getHeader('X-Icinga-WindowId', null);
-
-            if ($id === Window::UNDEFINED) {
-                $this->window = new Window($id);
-                $this->_response->setHeader('X-Icinga-WindowId', Window::generateId());
-            }
-        }
-    }
-
     protected function reloadCss()
     {
         $this->reloadCss = true;
@@ -308,42 +313,6 @@ class ActionController extends Zend_Controller_Action
         return $this->view->tabs;
     }
 
-    /**
-     * Translate a string
-     *
-     * Autoselects the module domain, if any, and falls back to the global one if no translation could be found.
-     *
-     * @param   string      $text       The string to translate
-     * @param   string|null $context    Optional parameter for context based translation
-     *
-     * @return  string                  The translated string
-     */
-    public function translate($text, $context = null)
-    {
-        return Translator::translate($text, $this->view->translationDomain, $context);
-    }
-
-    /**
-     * Translate a plural string
-     *
-     * @param string        $textSingular   The string in singular form to translate
-     * @param string        $textPlural     The string in plural form to translate
-     * @param string        $number         The number to get the plural or singular string
-     * @param string|null   $context        Optional parameter for context based translation
-     *
-     * @return string                       The translated string
-     */
-    public function translatePlural($textSingular, $textPlural, $number, $context = null)
-    {
-        return Translator::translatePlural(
-            $textSingular,
-            $textPlural,
-            $number,
-            $this->view->translationDomain,
-            $context
-        );
-    }
-
     protected function ignoreXhrBody()
     {
         if ($this->isXhr()) {
@@ -351,6 +320,18 @@ class ActionController extends Zend_Controller_Action
         }
     }
 
+    /**
+     * Set the interval (in seconds) at which the page should automatically refresh
+     *
+     * This may be adjusted based on the user's preferences. The result could be a
+     * lower or higher rate of the page's automatic refresh. If this is not desired,
+     * the only way to bypass this is to initialize the {@see ActionController::$autorefreshInterval}
+     * property or to set the `autorefreshInterval` property of the layout directly.
+     *
+     * @param int $interval
+     *
+     * @return $this
+     */
     public function setAutorefreshInterval($interval)
     {
         if (! is_int($interval) || $interval < 1) {
@@ -358,15 +339,22 @@ class ActionController extends Zend_Controller_Action
                 'Setting autorefresh interval smaller than 1 second is not allowed'
             );
         }
+
+        $user = $this->getRequest()->getUser();
+        if ($user !== null) {
+            $speed = (float) $user->getPreferences()->getValue('icingaweb', 'auto_refresh_speed', 1.0);
+            $interval = max(round($interval * $speed), min($interval, 5));
+        }
+
         $this->autorefreshInterval = $interval;
-        $this->_helper->layout()->autorefreshInterval = $interval;
+
         return $this;
     }
 
     public function disableAutoRefresh()
     {
         $this->autorefreshInterval = null;
-        $this->_helper->layout()->autorefreshInterval = null;
+
         return $this;
     }
 
@@ -483,9 +471,13 @@ class ActionController extends Zend_Controller_Action
             }
         }
 
-        if ($req->getParam('format') === 'pdf') {
-            $this->shutdownSession();
+        if ($this->autorefreshInterval !== null) {
+            $layout->autorefreshInterval = $this->autorefreshInterval;
+        }
+
+        if ($req->getParam('error_handler') === null && $req->getParam('format') === 'pdf') {
             $this->sendAsPdf();
+            $this->shutdownSession();
             exit;
         }
 
@@ -535,8 +527,12 @@ class ActionController extends Zend_Controller_Action
 
     protected function sendAsPdf()
     {
-        $pdf = new Pdf();
-        $pdf->renderControllerAction($this);
+        if (Module::exists('pdfexport')) {
+            $this->newSendAsPdf();
+        } else {
+            $pdf = new Pdf();
+            $pdf->renderControllerAction($this);
+        }
     }
 
     protected function shutdownSession()
