@@ -21,15 +21,39 @@ class StyleSheet
      */
     const DEFAULT_THEME = 'Icinga';
 
+    /**
+     * The name of the default theme mode
+     *
+     * @var string
+     */
+    const DEFAULT_MODE = 'none';
+
+    /**
+     * The themes that are compatible with the default theme
+     *
+     * @var array
+     */
+    const THEME_WHITELIST = [
+        'colorblind',
+        'high-contrast',
+        'Winter'
+    ];
+
+    /**
+     * Sequence that signals that a theme supports light mode
+     *
+     * @var string
+     */
+    const LIGHT_MODE_IDENTIFIER = '@light-mode:';
+
     /**
      * Array of core LESS files Web 2 sends to the client
      *
      * @var string[]
      */
-    protected static $lessFiles = array(
+    protected static $lessFiles = [
         '../application/fonts/fontello-ifont/css/ifont-embedded.css',
         'css/vendor/normalize.css',
-        'css/vendor/tipsy.css',
         'css/icinga/base.less',
         'css/icinga/badges.less',
         'css/icinga/mixins.less',
@@ -52,8 +76,11 @@ class StyleSheet
         'css/icinga/compat.less',
         'css/icinga/print.less',
         'css/icinga/responsive.less',
-        'css/icinga/modal.less'
-    );
+        'css/icinga/modal.less',
+        'css/icinga/audit.less',
+        'css/icinga/health.less',
+        'css/icinga/php-diff.less'
+    ];
 
     /**
      * Application instance
@@ -78,12 +105,15 @@ class StyleSheet
 
     /**
      * Create the StyleSheet
+     *
+     * @param bool $disableModes Disable replacing compiled Less colors with CSS var() function calls and don't inject
+     *                           light mode calls
      */
-    public function __construct()
+    public function __construct($disableModes = false)
     {
         $app = Icinga::app();
         $this->app = $app;
-        $this->lessCompiler = new LessCompiler();
+        $this->lessCompiler = new LessCompiler($disableModes);
         $this->pubPath = $app->getBaseDir('public');
         $this->collect();
     }
@@ -93,6 +123,12 @@ class StyleSheet
      */
     protected function collect()
     {
+        foreach ($this->app->getLibraries() as $library) {
+            foreach ($library->getCssAssets() as $lessFile) {
+                $this->lessCompiler->addLessFile($lessFile);
+            }
+        }
+
         foreach (self::$lessFiles as $lessFile) {
             $this->lessCompiler->addLessFile($this->pubPath . '/' . $lessFile);
         }
@@ -105,12 +141,6 @@ class StyleSheet
                     $this->lessCompiler->addModuleLessFile($moduleName, $lessFilePath);
                 }
             }
-
-            if ($module->requiresCss()) {
-                foreach ($module->getCssRequires() as $lessFilePath) {
-                    $this->lessCompiler->addModuleRequire($moduleName, $lessFilePath);
-                }
-            }
         }
 
         $themingConfig = $this->app->getConfig()->getSection('themes');
@@ -130,18 +160,32 @@ class StyleSheet
             }
         }
 
-        if ($theme && $theme !== self::DEFAULT_THEME) {
-            if (($pos = strpos($theme, '/')) !== false) {
-                $moduleName = substr($theme, 0, $pos);
-                $theme = substr($theme, $pos + 1);
-                if ($mm->hasLoaded($moduleName)) {
-                    $module = $mm->getModule($moduleName);
-                    $this->lessCompiler->setTheme($module->getCssDir() . '/themes/' . $theme . '.less');
-                }
+        if ($themePath = self::getThemeFile($theme)) {
+            if ($this->app->isCli() || is_file($themePath) && is_readable($themePath)) {
+                $this->lessCompiler->setTheme($themePath);
             } else {
-                $this->lessCompiler->setTheme($this->pubPath . '/css/themes/' . $theme . '.less');
+                $themePath = null;
+                Logger::warning(sprintf(
+                    'Theme "%s" set by user "%s" has not been found.',
+                    $theme,
+                    ($user = Auth::getInstance()->getUser()) !== null ? $user->getUsername() : 'anonymous'
+                ));
             }
         }
+
+        if (! $themePath || in_array($theme, self::THEME_WHITELIST, true)) {
+            $this->lessCompiler->addLessFile($this->pubPath . '/css/icinga/login-orbs.less');
+        }
+
+        $mode = 'none';
+        if ($user = Auth::getInstance()->getUser()) {
+            $file = $themePath !== null ? @file_get_contents($themePath) : false;
+            if (! $file || strpos($file, self::LIGHT_MODE_IDENTIFIER) !== false) {
+                $mode = $user->getPreferences()->getValue('icingaweb', 'theme_mode', self::DEFAULT_MODE);
+            }
+        }
+
+        $this->lessCompiler->setThemeMode($this->pubPath . '/css/modes/'. $mode . '.less');
     }
 
     /**
@@ -152,6 +196,8 @@ class StyleSheet
     public static function forPdf()
     {
         $styleSheet = new self();
+        $styleSheet->lessCompiler->setTheme(null);
+        $styleSheet->lessCompiler->setThemeMode($styleSheet->pubPath . '/css/modes/none.less');
         $styleSheet->lessCompiler->addLessFile($styleSheet->pubPath . '/css/pdf/pdfprint.less');
         // TODO(el): Caching
         return $styleSheet;
@@ -177,15 +223,17 @@ class StyleSheet
      *
      * Does not cache the stylesheet if the HTTP header Cache-Control or Pragma is set to no-cache.
      *
-     * @param   bool    $minified   Whether to compress the stylesheet
+     * @param bool $minified     Whether to compress the stylesheet
+     * @param bool $disableModes Disable replacing compiled Less colors with CSS var() function calls and don't inject
+     *                           light mode calls
      */
-    public static function send($minified = false)
+    public static function send($minified = false, $disableModes = false)
     {
-        $styleSheet = new self();
+        $styleSheet = new self($disableModes);
 
         $request = $styleSheet->app->getRequest();
         $response = $styleSheet->app->getResponse();
-        $response->setHeader('Cache-Control', 'public', true);
+        $response->setHeader('Cache-Control', 'private,no-cache,must-revalidate', true);
 
         $noCache = $request->getHeader('Cache-Control') === 'no-cache' || $request->getHeader('Pragma') === 'no-cache';
 
@@ -229,4 +277,32 @@ class StyleSheet
             return IcingaException::describe($e);
         }
     }
+
+    /**
+     * Get the path to the current LESS theme file
+     *
+     * @param $theme
+     *
+     * @return  string|null Return null if self::DEFAULT_THEME is set as theme, path otherwise
+     */
+    public static function getThemeFile($theme)
+    {
+        $app = Icinga::app();
+
+        if ($theme && $theme !== self::DEFAULT_THEME) {
+            if (($pos = strpos($theme, '/')) !== false) {
+                $moduleName = substr($theme, 0, $pos);
+                $theme = substr($theme, $pos + 1);
+                if ($app->getModuleManager()->hasLoaded($moduleName)) {
+                    $module = $app->getModuleManager()->getModule($moduleName);
+
+                    return $module->getCssDir() . '/themes/' . $theme . '.less';
+                }
+            } else {
+                return $app->getBaseDir('public') . '/css/themes/' . $theme . '.less';
+            }
+        }
+
+        return null;
+    }
 }
