@@ -5,10 +5,10 @@ namespace Icinga\Web;
 
 use Closure;
 use Icinga\Application\Icinga;
+use ipl\I18n\Translation;
 use Zend_View_Abstract;
 use Icinga\Authentication\Auth;
 use Icinga\Exception\ProgrammingError;
-use Icinga\Util\Translator;
 
 /**
  * Icinga view
@@ -54,18 +54,13 @@ use Icinga\Util\Translator;
  */
 class View extends Zend_View_Abstract
 {
+    use Translation;
+
     /**
      * Charset to be used - we only support UTF-8
      */
     const CHARSET = 'UTF-8';
 
-    /**
-     * Flag to register stream wrapper
-     *
-     * @var bool
-     */
-    private $useViewStream = false;
-
     /**
      * Registered helper functions
      */
@@ -86,13 +81,6 @@ class View extends Zend_View_Abstract
      */
     public function __construct($config = array())
     {
-        $this->useViewStream = (bool) ini_get('short_open_tag') ? false : true;
-        if ($this->useViewStream) {
-            if (!in_array('zend.view', stream_get_wrappers())) {
-                stream_wrapper_register('zend.view', '\Icinga\Web\ViewStream');
-            }
-        }
-
         $config['helperPath']['Icinga\\Web\\View\\Helper\\'] = Icinga::app()->getLibraryDir('Icinga/Web/View/Helper');
 
         parent::__construct($config);
@@ -192,21 +180,6 @@ class View extends Zend_View_Abstract
         );
     }
 
-    public function translate($text, $context = null)
-    {
-        return Translator::translate($text, $this->translationDomain, $context);
-    }
-
-    /**
-     * Translate a plural string
-     *
-     * @see Translator::translatePlural()
-     */
-    public function translatePlural($textSingular, $textPlural, $number, $context = null)
-    {
-        return Translator::translatePlural($textSingular, $textPlural, $number, $this->translationDomain, $context);
-    }
-
     /**
      * Load helpers
      */
@@ -258,11 +231,8 @@ class View extends Zend_View_Abstract
             // Exporting global variables to view scripts:
             $$k = $v;
         }
-        if ($this->useViewStream) {
-            include 'zend.view://' . func_get_arg(0);
-        } else {
-            include func_get_arg(0);
-        }
+
+        include func_get_arg(0);
     }
 
     /**
