@@ -83,7 +83,6 @@ class ListController extends Controller
             'host_check_command',
             'host_next_update'
         ), $this->addColumns()));
-        $this->applyRestriction('monitoring/filter/objects', $hosts);
 
         $this->setupPaginationControl($hosts);
         $this->setupSortControl(array(
@@ -110,11 +109,8 @@ class ListController extends Controller
         ));
         $this->applyRestriction('monitoring/filter/objects', $stats);
 
-        $summary = $hosts->getQuery()->queryServiceProblemSummary();
-
         $this->view->hosts = $hosts;
         $this->view->stats = $stats;
-        $this->view->summary = $summary->fetchPairs();
     }
 
     /**
@@ -165,7 +161,6 @@ class ListController extends Controller
             'service_check_command',
             'service_next_update'
         ), $this->addColumns()));
-        $this->applyRestriction('monitoring/filter/objects', $services);
 
         $this->setupPaginationControl($services);
         $this->setupSortControl(array(
@@ -243,7 +238,6 @@ class ListController extends Controller
             'host_display_name',
             'service_display_name'
         ));
-        $this->applyRestriction('monitoring/filter/objects', $downtimes);
 
         $this->setupPaginationControl($downtimes);
         $this->setupSortControl(array(
@@ -293,7 +287,6 @@ class ListController extends Controller
             'service_description',
             'service_display_name'
         ));
-        $this->applyRestriction('monitoring/filter/objects', $notifications);
 
         $this->setupPaginationControl($notifications);
         $this->setupSortControl(array(
@@ -328,7 +321,6 @@ class ListController extends Controller
             'contact_notify_service_timeperiod',
             'contact_notify_host_timeperiod'
         ));
-        $this->applyRestriction('monitoring/filter/objects', $contacts);
 
         $this->setupPaginationControl($contacts);
         $this->setupSortControl(array(
@@ -408,7 +400,6 @@ class ListController extends Controller
             'contactgroup_alias',
             'contact_count'
         ));
-        $this->applyRestriction('monitoring/filter/objects', $contactGroups);
 
         $this->setupPaginationControl($contactGroups);
         $this->setupSortControl(array(
@@ -449,7 +440,6 @@ class ListController extends Controller
             'host_display_name',
             'service_display_name'
         ));
-        $this->applyRestriction('monitoring/filter/objects', $comments);
 
         $this->setupPaginationControl($comments);
         $this->setupSortControl(
@@ -499,7 +489,6 @@ class ListController extends Controller
             'services_warning_handled',
             'services_warning_unhandled'
         ));
-        $this->applyRestriction('monitoring/filter/objects', $serviceGroups);
 
         $this->setupPaginationControl($serviceGroups);
         $this->setupSortControl(array(
@@ -539,7 +528,6 @@ class ListController extends Controller
             'services_warning_handled',
             'services_warning_unhandled'
         ));
-        $this->applyRestriction('monitoring/filter/objects', $serviceGroups);
         $this->filterQuery($serviceGroups);
 
         $this->setupSortControl(array(
@@ -584,7 +572,6 @@ class ListController extends Controller
             'services_warning_handled',
             'services_warning_unhandled'
         ));
-        $this->applyRestriction('monitoring/filter/objects', $hostGroups);
 
         $this->setupPaginationControl($hostGroups);
         $this->setupSortControl(array(
@@ -623,7 +610,6 @@ class ListController extends Controller
             'hosts_unreachable_unhandled',
             'hosts_up'
         ]);
-        $this->applyRestriction('monitoring/filter/objects', $hostGroups);
         $this->filterQuery($hostGroups);
 
         $this->setupSortControl([
@@ -657,7 +643,6 @@ class ListController extends Controller
             'type'
         ));
 
-        $this->applyRestriction('monitoring/filter/objects', $query);
         $this->view->history = $query;
 
         $this->setupSortControl(array(
@@ -684,7 +669,6 @@ class ListController extends Controller
             'service_output',
             'service_state'
         ));
-        $this->applyRestriction('monitoring/filter/objects', $query);
         $this->filterQuery($query);
         $filter = (bool) $this->params->shift('problems', false) ? Filter::where('service_problem', 1) : null;
 
@@ -751,7 +735,13 @@ class ListController extends Controller
             'problems', // servicegridAction()
             'flipped' // servicegridAction()
         ));
+
+        if ($this->params->get('format') !== 'sql' || $this->hasPermission('config/authentication/roles/show')) {
+            $this->applyRestriction('monitoring/filter/objects', $dataView);
+        }
+
         $this->handleFormatRequest($dataView);
+
         return $dataView;
     }
 
