@@ -573,7 +573,10 @@ abstract class IdoQuery extends DbQuery
         $column = $subQuery->aliasToColumnName($alias);
         if (isset($this->caseInsensitiveColumns[$subQuery->aliasToTableName($alias)][$alias])) {
             $column = 'LOWER( ' . $column . ' )';
+            $subQueryFilter->setColumn($column);
             $subQueryFilter->setExpression(array_map('strtolower', (array) $subQueryFilter->getExpression()));
+        } else {
+            $subQueryFilter->setColumn($column);
         }
 
         $additional = null;
@@ -613,16 +616,18 @@ abstract class IdoQuery extends DbQuery
             }
         }
 
-        if ($and || $negate && ! $and) {
+        if ($and || $negate) {
             // Having is only required for AND and != filters,
             // e.g. hostgroup_name=(ping&linux), hostgroup_name!=ping, hostgroup_name!=(ping|linux)
             $groups = $subQuery->getGroup();
-            $group = $groups[0];
-            $group = preg_replace('/(?<=^|\s)\w+(?=\.)/', 'sub_$0', $group);
+            if (! empty($groups)) {
+                $group = $groups[0];
+                $group = preg_replace('/(?<=^|\s)\w+(?=\.)/', 'sub_$0', $group);
 
-            $cnt = count($expr);
+                $cnt = count($expr);
 
-            $subQuery->select()->having("COUNT(DISTINCT $group) >= $cnt");
+                $subQuery->select()->having("COUNT(DISTINCT $group) >= $cnt");
+            }
         }
 
         $subQueryFilter->setColumn(preg_replace(
@@ -653,10 +658,6 @@ abstract class IdoQuery extends DbQuery
         $exists = new FilterExpression($negate ? 'NOT EXISTS' : 'EXISTS', '', new Zend_Db_Expr($subQuery));
 
         if ($additional !== null) {
-            $alias = $additional->getColumn();
-            $this->requireColumn($alias);
-            $additional->setColumn($this->aliasToColumnName($alias));
-
             return Filter::matchAll($exists, $additional);
         }
 
@@ -666,10 +667,6 @@ abstract class IdoQuery extends DbQuery
     protected function requireFilterColumns(Filter $filter)
     {
         if ($filter instanceof FilterExpression) {
-            if ($filter->getExpression() === '*') {
-                return; // Wildcard only filters are ignored so stop early here to avoid joining a table for nothing
-            }
-
             $alias = $filter->getColumn();
 
             $virtualTable = $this->aliasToTableName($alias);
@@ -775,10 +772,6 @@ abstract class IdoQuery extends DbQuery
 
     public function where($condition, $value = null)
     {
-        if ($value === '*') {
-            return $this; // Wildcard only filters are ignored so stop early here to avoid joining a table for nothing
-        }
-
         $this->requireColumn($condition);
         $col = $this->getMappedField($condition);
         if ($col === null) {
@@ -1200,14 +1193,30 @@ abstract class IdoQuery extends DbQuery
 
         $this->customVars[strtolower($customvar)] = $alias;
 
-        if ($this->hasJoinedVirtualTable('services')) {
-            $leftcol = 's.' . $type . '_object_id';
-        } elseif ($type === 'service') {
-            $this->requireVirtualTable('services');
-            $leftcol = 's.service_object_id';
-        } else {
-            $this->requireVirtualTable('hosts');
-            $leftcol = 'h.host_object_id';
+        if ($type === 'host') {
+            if ($this instanceof ServicecommentQuery
+                || $this instanceof ServicedowntimeQuery
+                || $this instanceof ServicecommenthistoryQuery
+                || $this instanceof ServicedowntimestarthistoryQuery
+                || $this instanceof ServiceflappingstarthistoryQuery
+                || $this instanceof ServicegroupQuery
+                || $this instanceof ServicenotificationQuery
+                || $this instanceof ServicestatehistoryQuery
+                || $this instanceof ServicestatusQuery
+            ) {
+                $this->requireVirtualTable('services');
+                $leftcol = 's.host_object_id';
+            } else {
+                $leftcol = 'ho.object_id';
+                if (! $this->hasJoinedTable('ho')) {
+                    $this->requireVirtualTable('hosts');
+                }
+            }
+        } else { // $type === 'service'
+            $leftcol = 'so.object_id';
+            if (! $this->hasJoinedTable('so')) {
+                $this->requireVirtualTable('services');
+            }
         }
 
         $mapped = $this->getMappedField($leftcol);
