@@ -307,8 +307,8 @@ class Perfdata
         $parts = explode(';', $this->perfdataValue);
 
         $matches = array();
-        if (preg_match('@^(-?\d+(\.\d+)?)([a-zA-Z%°]{1,2})$@u', $parts[0], $matches)) {
-            $this->unit = strtolower($matches[3]);
+        if (preg_match('@^(-?(?:\d+)?(?:\.\d+)?)([a-zA-Z%°]{1,3})$@u', $parts[0], $matches)) {
+            $this->unit = strtolower($matches[2]);
             $this->value = self::convert($matches[1], $this->unit);
         } else {
             $this->value = self::convert($parts[0]);
@@ -425,9 +425,6 @@ class Perfdata
         $pieChart = new InlinePie($data, $this);
         $pieChart->setColors(array('#44bb77', '#ffaa44', '#ff5566', '#ddccdd'));
 
-        if (Zend_Controller_Front::getInstance()->getRequest()->isXmlHttpRequest()) {
-            $pieChart->disableNoScript();
-        }
         return $pieChart;
     }
 
@@ -436,6 +433,10 @@ class Perfdata
      */
     protected function format($value)
     {
+        if ($value === null) {
+            return null;
+        }
+
         if ($value instanceof ThresholdRange) {
             if ($value->getMin()) {
                 return (string) $value;
@@ -457,7 +458,7 @@ class Perfdata
         if ($this->isTemperature()) {
             return (string)$value . strtoupper($this->unit);
         }
-        return number_format($value, 2);
+        return number_format($value, 2) . ($this->unit !== null ? ' ' . $this->unit : '');
     }
 
     /**
