@@ -1,6 +1,5 @@
 /*! Icinga Web 2 | (c) 2015 Icinga Development Team | GPLv2+ */
 
-// @TODO(el): https://dev.icinga.com/issues/10646
 (function(Icinga, $) {
 
     'use strict';
@@ -9,14 +8,18 @@
 
     var ApplicationState = function (icinga) {
         Icinga.EventListener.call(this, icinga);
-        this.on('rendered', this.onRendered, this);
-        this.on('rendered', '#application-state-summary', this.onSummaryRendered, this);
+        this.on('rendered', '#layout', this.onRendered, this);
         this.icinga = icinga;
     };
 
     ApplicationState.prototype = new Icinga.EventListener();
 
     ApplicationState.prototype.onRendered = function(e) {
+        if (e.currentTarget !== e.target) {
+            // Nested containers are ignored
+            return;
+        }
+
         if (! $('#application-state').length
             && ! $('#login').length
             && ! $('#guest-error').length
@@ -32,13 +35,6 @@
         }
     };
 
-    ApplicationState.prototype.onSummaryRendered = function(e) {
-        var height = $(this).height();
-
-        $('#sidebar').css('bottom', height);
-        $('#main').css('bottom', height);
-    };
-
     Icinga.Behaviors.ApplicationState = ApplicationState;
 
 })(Icinga, jQuery);
